#include "headers.h"
#include "user.h"

static uint32_t build_privs(SV **sp, SV **mark, I32 ax, I32 items) {
	uint32_t ret = PRIVS_EMPTY;
	int i,j,p;

	for(i = 1; i < items; ++i) {
		STRLEN n_a;
		char *str = SvPV(ST(i),n_a);
		if(str == NULL)
			continue;
		for(j = 0, p = -1; priv_text[j].text != NULL && p != -1; ++j) {
			if(strcasecmp(str,priv_text[j].text) == 0)
				p = priv_text[j].priv;
		}
		if(p != -1)
			ret |= PRIV_TO_MASK(p);
	}

	return ret;
}

MODULE = Arisa::User  PACKAGE = Arisa
PROTOTYPES: ENABLE

Arisa::User
user_find(name = NULL)
	char *name
CODE:
	RETVAL = user_find(name);
OUTPUT:
	RETVAL

void
users()
PREINIT:
	int i;
PPCODE:
	LOCK(global);
	for(i = 0; i < global->no_users; ++i) {
		user_t *u = global->users[i];
		LOCK(u);
		if(u->deleted == 0)
			XPUSHs(sv_2mortal(
				create_ptr_object("Arisa::User",u)));
		UNLOCK(u);
	}
	UNLOCK(global);

Arisa::User
user_add(name = NULL)
	char *name
CODE:
	RETVAL = NULL;
	if(name != NULL) {
		user_t *u;
		
		u = alloc_user(name);
		
		if(user_add(u) == 0) {
			RETVAL = u;
		} else {
			free_user(u);
		}
	}
OUTPUT:
	RETVAL

	
#*******************************
MODULE = Arisa::User  PACKAGE = Arisa::User
#*******************************

SV*
delete(user)
	Arisa::User user
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(user)) {
		if(user_del(user) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

void
dirs(user, ...)
	Arisa::User user
PPCODE:
	if(user_valid(user)) {
		LOCK(user);
		LOAD_RET_STRARR(user->dirs,user->no_dirs);
		UNLOCK(user);
	}
	
SV*
grant(u,...)
	Arisa::User u
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(u)) {
		uint32_t privs = build_privs(sp,mark,ax,items);
		if(privs != PRIVS_EMPTY) {
			LOCK(u);
			user_set_privs(u,u->uprivs | privs);
			UNLOCK(u);
			RETVAL = &PL_sv_yes;
		}
	}
OUTPUT:
	RETVAL

void
hosts(user, ...)
	Arisa::User user
PPCODE:
	if(user_valid(user)) {
		LOCK(user);
		LOAD_RET_STRARR(user->hosts,user->no_hosts);
		UNLOCK(user);
	}
	
void
info(u)
	Arisa::User u
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(user_valid(u)) {
		hv = newHV();
		LOCK(u);
		hv_store(hv,"last_logon",10,newSVuv(u->last_logon),0);
		UNLOCK(u);
	}
	RET_REF(hv);
	
SV*
name(user, new_name = NULL)
	Arisa::User user
	char *new_name
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(user)) {
		if(new_name != NULL) {
			if(user_rename(user,new_name) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			LOCK(user);
			RETVAL = new_pv(user->username);
			UNLOCK(user);
		}
	}
OUTPUT:
	RETVAL


void
packlists(u, ...)
	Arisa::User u
PPCODE:
	if(user_valid(u)) {
		int i;

		if(items > 1) {
			packlist_t **pls = NULL;
			int no_pls = 0;
			int j,found;
			
			SV *sv = ST(1);
			if(SvTYPE(sv) == SVt_RV)
				sv = SvRV(sv);
			if(SvTYPE(sv) == SVt_PVAV) {
				AV *av = (AV*)sv;
				for(i = 0; i < av_len(av); ++i) {
					SV *v = *(av_fetch(av,i,0));
					packlist_t *pl = fetch_object_ptr(v);
					if(pl != NULL)
						PTRARR_ADD(&pls,&no_pls,pl);
				}
			}

			LOCK(global);
			for(i = 0; i < no_pls; ++i) {
				for(j = 0, found = 0; j < global->no_packlists && !found; ++j) {
					if(global->packlists[j] == pls[i]) {
						found = 1;
						LOCK(pls[i]);
						if(pls[i]->deleted != 0) {
							UNLOCK(pls[i]);
							pls[i] = NULL;
						} else {
							UNLOCK(pls[i]);
						}
					}
				}
				if(!found)
					pls[i] = NULL;
			}

			PTRARR_DEL(&pls,&no_pls,NULL);
			
			LOCK(u);
			if(u->packlists != NULL)
				xfree(u->packlists);
			u->packlists = pls;
			u->no_packlists = no_pls;
			UNLOCK(u);

			UNLOCK(global);
		}

		LOCK(u);
		for(i = 0; i < u->no_packlists; ++i) {
			XPUSHs(sv_2mortal(
				create_ptr_object("Arisa::Packlist",u->packlists[i])));
		}
		UNLOCK(u);
	}	
	
void
privileges(u, ...)
	Arisa::User u
PPCODE:
	if(user_valid(u)) {
		uint32_t privs;
		int i;

		LOCK(u);
		if(items > 1) {
			privs = build_privs(sp,mark,ax,items);
			user_set_privs(u,privs);
		}
		for(i = 0; priv_text[i].text != NULL; ++i) {
			if(PRIV_ISSET(u,priv_text[i].priv))
				XPUSHs(sv_2mortal(new_pv(priv_text[i].text)));
		}
		UNLOCK(u);
	}

SV*
revoke(u, ...)
	Arisa::User u
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(u)) {
		uint32_t privs = build_privs(sp,mark,ax,items);
		if(privs != PRIVS_EMPTY) {
			LOCK(u);
			user_set_privs(u,u->uprivs & (~privs));
			UNLOCK(u);
			RETVAL = &PL_sv_yes;
		}
	}
OUTPUT:
	RETVAL

SV*
set_password(user, password)
	Arisa::User user
	char *password
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(user) && password != NULL) {
		if(user_set_password(user,password) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL
	
void
settings(user, ...)
	Arisa::User user
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(user_valid(user)) {
		hv = newHV();
		perl_apply_settings(user_apply_setting,user);
		perl_read_settings(user_read_settings,user,hv);
	}
	RET_REF(hv);

SV*
test_password(user, password)
	Arisa::User user
	char *password
CODE:
	RETVAL = &PL_sv_no;
	if(user_valid(user) && password != NULL) {
		if(user_check_password(user,password))
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL
